@@ -1,23 +1,31 @@
 geks_matrix <- function(index, p, q, product, n, nper, window, na.rm) {
   # making base prices/quantities is the slowest part of the calculation;
   # the algorithm calculates the lower-triangular part of the GEKS matrix
-  # to avoid making relatives with different bases, then uses the 
+  # to avoid making relatives with different bases, then uses the
   # time-reversal property of the 'index' function
   rows <- seq_len(nper)
   lt <- lapply(rows, function(i) {
-    ans <- if (i < max(window - n, 2L)) {
-      # only the last n + 1 rows are needed for each window, 
+    if (i < max(window - n, 2L)) {
+      # only the last n + 1 rows are needed for each window,
       # so pad the top rows left of the diagonal with NA
-      rep_len(NA_real_, i - 1L)
+      ans <- rep_len(NA_real_, i - 1L)
     } else {
       # matching is only done for the lower-triangular part of the matrix
       # match products for window - 1 periods left of the diagonal
       # to minimize the number of back prices to find
-      js <- seq(to = i - 1L, length.out = min(window, i) - 1L)
-      m <- .mapply(match, list(product[js], product[i]), list(incomparables = NA))
+      js <- seq.int(to = i - 1L, length.out = min(window, i) - 1L)
+      m <- .mapply(
+        match,
+        list(product[js], product[i]),
+        list(incomparables = NA)
+      )
       bp <- .mapply(`[`, list(p[i], m), list())
       bq <- .mapply(`[`, list(q[i], m), list())
-      .mapply(index, list(p1 = p[js], p0 = bp, q1 = q[js], q0 = bq), list(na.rm = na.rm))
+      ans <- .mapply(
+        index,
+        list(p1 = p[js], p0 = bp, q1 = q[js], q0 = bq),
+        list(na.rm = na.rm)
+      )
     }
     # add the diagonal at the end and pad with NAs
     ans <- c(unlist(ans, use.names = FALSE), 1)
@@ -35,44 +43,58 @@ geks_matrix <- function(index, p, q, product, n, nper, window, na.rm) {
 
 geks <- function(f) {
   f <- match.fun(f)
-  # return function
-  function(p, q, period, product, window = nlevels(period), n = window - 1L, na.rm = FALSE) {
+
+  function(p, q, period, product,
+           window = nlevels(period), n = window - 1L, na.rm = FALSE) {
     if (different_lengths(p, q, period, product)) {
-      stop(gettext("'p', 'q', 'period', and 'product' must be the same length"))
+      stop("'p', 'q', 'period', and 'product' must be the same length")
     }
+
     period <- as.factor(period)
     nper <- nlevels(period)
-    if (!nper) return(list())
+
+    if (nper == 0L) {
+      return(list())
+    }
+
     window <- as.integer(window[1L])
     if (window < 2L) {
-      stop(gettext("'window' must be greater than or equal to 2"))
+      stop("'window' must be greater than or equal to 2")
     }
     if (window > nper) {
-      stop(gettext("'window' must be less than or equal to the number of levels in 'period'"))
+      stop("'window' must be less than or equal to the number of levels in",
+           " 'period'")
     }
+
     n <- as.integer(n[1L])
     if (n < 1L) {
-      stop(gettext("'n' must be greater than or equal to 1"))
+      stop("'n' must be greater than or equal to 1")
     }
     if (n > window - 1L) {
-      stop(gettext("'n' must be less than or equal to 'window' minus 1"))
+      stop("'n' must be less than or equal to 'window' minus 1")
     }
+
     p <- split(p, period)
     q <- split(q, period)
     product <- as.factor(product)
     attributes(product) <- NULL # faster to match on numeric codes
     product <- split(product, period)
-    if (max(vapply(product, anyDuplicated, numeric(1L), incomparables = NA))) {
-      warning(gettext("there are duplicated period-product pairs"))
+    if (duplicate_products(product)) {
+      warning("there are duplicated period-product pairs")
     }
+
     mat <- geks_matrix(f, p, q, product, n, nper, window, na.rm)
     rows <- seq_len(window) - 1L
     # only the last n + 1 indexes in each window need to be kept
-    cols <- seq(window - n, window) - 1L 
+    cols <- seq.int(window - n, window) - 1L
     res <- vector("list", nper - window + 1L)
     # move down the diagonal to make the geks index
     for (i in seq_along(res)) {
-      index <- apply(mat[rows + i, cols + i, drop = FALSE], 2L, geometric_mean, na.rm = na.rm)
+      index <- apply(
+        mat[rows + i, cols + i, drop = FALSE], 2L,
+        geometric_mean,
+        na.rm = na.rm
+      )
       res[[i]] <- index[-1L] / index[-length(index)]
     }
     res
